%% batch example of the GEFF framework, proposed in (Abbas et al, NeuroImage 2020),
%% to maximize subject and task fingerprint in human functional connectomes
%% 
%% Kausar Abbas & Joaquín Goñi, Purdue University
%% version 1.0. July 21, 2020
%
%% PLEASE CITE US!
% If you are using this code for your research, please kindly cite us:
%% GEFF: Graph Embedding for Functional Fingerprinting by Abbas et al.
% NeuroImage (in press) 2020. 
% https://www.sciencedirect.com/science/article/pii/S1053811920306674

%% initialize environment
close all;
clearvars
clc

%% changing default fontsize
fontsize = 24;
set(0,'DefaultAxesFontName','Times New Roman');
set(0,'DefaultAxesFontSize',fontsize-2);

set(0,'DefaultTextFontname','Times New Roman');
set(0,'DefaultTextFontSize',fontsize);

%% Load FC (functional connectivity) resting-state data. This is a sample data.
% The sample data contains three tasks for 20 subjects, test and retest
% Tasks included are: resting-state, gambling and relational 
% (For more details see HCP papers)

% FC_test: contains test FCs for all three tasks
%          1. dimensions are brain regions x brain regions x number of FCs
%          2. number of FCs = number of subjects x number of tasks = 20 * 3 = 60
%          3. FCs are ordered such that the first three FCs are the
%          resting-state, gambling and relational FCs for the first subject and so on ...

% FC_retest: contains retest FCs for the same subjects and tasks as in FC_test and in the same order

load FC_test_retest.mat
FC_test = single(FC_test);
FC_retest = single(FC_retest);

%% Configuration
configs.numROIs = 374; % Glasser parcellation. Number of brain regions
configs.mask_ut = triu(true(configs.numROIs,configs.numROIs),1); % upper triangular mask
configs.numTasks = 3; % maximum number of tasks in this dataset (resting-state, gambling, and relational)
configs.Task_labels = {'resting-state','gambling','relational'};
configs.numSubjs = 20; % maximum number of subjects in this dataset
configs.numEdges = nnz(configs.mask_ut);

%% create 2D input matrix from original FCs (see Fig.1 in Abbas et al. 2020) 
% Since FCs are symmetric matrices, only the upper triangular part is kept
FC_test_vec = nan(configs.numEdges,size(FC_test,3));
FC_retest_vec = nan(configs.numEdges,size(FC_retest,3));

for i = 1:size(FC_test,3)
    FC = squeeze(FC_test(:,:,i));
    FC_test_vec(:,i) = FC(configs.mask_ut);
    
    FC = squeeze(FC_retest(:,:,i));
    FC_retest_vec(:,i) = FC(configs.mask_ut);
end

%% Subject-Identification (SI)
% Specify indeces of the tasks chosen for the Learning Stage
index_learning = [1 2]; % resting-state and relational
num_learning_tasks = length(index_learning);

Task_nums = 1:configs.numTasks;
index_remaining_tasks = Task_nums(~ismember(Task_nums,index_learning));
num_remaining_tasks = length(index_remaining_tasks);

% Create learning and validation matrices for both Within- and Across-Task subject identification
% Also, create corresponding labels for these matrices
learning_matrix = nan(configs.numEdges,num_learning_tasks*configs.numSubjs);
validation_matrix_within = nan(configs.numEdges,num_learning_tasks*configs.numSubjs);
validation_matrix_across = nan(configs.numEdges,num_remaining_tasks*configs.numSubjs);

labels_learning = nan(1,num_learning_tasks*configs.numSubjs);
labels_validation_within = nan(1,num_learning_tasks*configs.numSubjs);
labels_validation_across = nan(1,num_remaining_tasks*configs.numSubjs);

for s = 1:configs.numSubjs

    % The learning matrix
    learning_matrix(:,(s-1)*num_learning_tasks+1:(s-1)*num_learning_tasks+num_learning_tasks) = ...
        FC_test_vec(:,(s-1)*configs.numTasks+index_learning);
    labels_learning(1,(s-1)*num_learning_tasks+1:(s-1)*num_learning_tasks+num_learning_tasks) = s;

    % Within-Task validation matrix and labels
    validation_matrix_within(:,(s-1)*num_learning_tasks+1:(s-1)*num_learning_tasks+num_learning_tasks) = ...
        FC_retest_vec(:,(s-1)*configs.numTasks+index_learning);
    labels_validation_within(1,(s-1)*num_learning_tasks+1:(s-1)*num_learning_tasks+num_learning_tasks) = s;
    
    % Across-Task validation matrix and labels
    validation_matrix_across(:,(s-1)*num_remaining_tasks+1:(s-1)*num_remaining_tasks+num_remaining_tasks) = ...
        FC_retest_vec(:,(s-1)*configs.numTasks+index_remaining_tasks);
    labels_validation_across(1,(s-1)*num_remaining_tasks+1:(s-1)*num_remaining_tasks+num_remaining_tasks) = s;

end

% Compute Subject-Identification (SI) rates using Orig FCs and GEFF
% Within-Task Subject Identification
rate_orig_within = f_identify_subject_orig_FCs(learning_matrix,validation_matrix_within,labels_learning,labels_validation_within,configs);
[rate_GEFF_within,rate_null_GEFF_within] = ...
    f_identify_subject_GEFF(learning_matrix,validation_matrix_within,labels_learning,labels_validation_within,configs);

% Across-Task Subject Identification
rate_orig_across = f_identify_subject_orig_FCs(learning_matrix,validation_matrix_across,labels_learning,labels_validation_across,configs);
[rate_GEFF_across,rate_null_GEFF_across] = ...
    f_identify_subject_GEFF(learning_matrix,validation_matrix_across,labels_learning,labels_validation_across,configs);

% Plot the figure
cmap = [0 0.4470 0.7410;
        0.8500 0.3250 0.0980;
        0.9290 0.6940 0.1250;
        0.4660 0.6740 0.1880];
    
h = figure('Name','Subject Identification','Units','normalized','Position',[0 0 1 1]);
x = [1 size(rate_GEFF_within,1)];

subplot(1,2,1);
for i = 1:num_learning_tasks
    plot(x,[rate_orig_within(i) rate_orig_within(i)],'--','LineWidth',1,'Color',cmap(i,:)); hold on;
    l{(i-1)*3+1} = strcat('Orig FCs','---',configs.Task_labels{index_learning(i)});
    
    plot(rate_GEFF_within(:,i),'LineWidth',2,'Color',cmap(i,:));
    l{(i-1)*3+2} = strcat('GEFF','---',configs.Task_labels{index_learning(i)});
    
    plot(rate_null_GEFF_within(:,i),'-*','LineWidth',1,'Color',cmap(i,:));
    l{(i-1)*3+3} = strcat('Null Model (GEFF)','---',configs.Task_labels{index_learning(i)});
end
axis([1 size(rate_GEFF_within,1) 0 1]);
xlabel('Eigenspace Dimensionality');
ylabel('Subject Identification Rate');
title('Within-Task (resting-state and gambling)');
legend(l,'Location','southeast');

subplot(1,2,2);
clear l
for i = 1:num_remaining_tasks
    plot(x,[rate_orig_across(i) rate_orig_across(i)],'--','LineWidth',1,'Color',cmap(i,:)); hold on;
    l{(i-1)*3+1} = strcat('Orig FCs','---',configs.Task_labels{index_remaining_tasks(i)});
    
    plot(rate_GEFF_across(:,i),'LineWidth',2,'Color',cmap(i,:));
    l{(i-1)*3+2} = strcat('GEFF','---',configs.Task_labels{index_remaining_tasks(i)});
    
    plot(rate_null_GEFF_across(:,i),'-*','LineWidth',1,'Color',cmap(i,:));
    l{(i-1)*3+3} = strcat('Null Model (GEFF)','---',configs.Task_labels{index_remaining_tasks(i)});
end
axis([1 size(rate_GEFF_across,1) 0 1]);
xlabel('Eigenspace Dimensionality');
ylabel('Subject Identification Rate');
title('Across-Task (LS: resting-state and gambling; VS: relational)');
legend(l,'Location','southeast');

%% Task-Identifcation (TI)
clear rate_* label_* l learning_matrix validation_matrix_*
% Re-arrange the FCs in task-wise order to keep the code consistent
% Now the FCs would be ordered by task:
% First 20 FCs would belong to resting-state, next 20 to gambling,
% and the last 20 to the relational task
FCt_test_vec = nan(configs.numEdges,size(FC_test,3));
FCt_retest_vec = nan(configs.numEdges,size(FC_retest,3));

for t = 1:configs.numTasks
    index = (t-1)*configs.numSubjs+1:(t-1)*configs.numSubjs+configs.numSubjs;
    FCt_test_vec(:,index) = FC_test_vec(:,t:configs.numTasks:end);
    FCt_retest_vec(:,index) = FC_retest_vec(:,t:configs.numTasks:end);
end

% Specify indeces of the subjects chosen for the Learning Stage
index_learning = 1:16; % resting-state and relational
num_learning_subjs = length(index_learning);

Subj_nums = 1:configs.numSubjs;
index_remaining_subjs = Subj_nums(~ismember(Subj_nums,index_learning));
num_remaining_subjs = length(index_remaining_subjs);

% Create learning and validation matrices for both Within-Learning- and Different-Subject task identification
% Also, create corresponding labels for these matrices
learning_matrix = nan(configs.numEdges,num_learning_subjs*configs.numTasks);
validation_matrix_within = nan(configs.numEdges,num_learning_subjs*configs.numTasks);
validation_matrix_different = nan(configs.numEdges,num_remaining_subjs*configs.numTasks);

labels_learning = nan(1,num_learning_subjs*configs.numTasks);
labels_validation_within = nan(1,num_learning_subjs*configs.numTasks);
labels_validation_different = nan(1,num_remaining_subjs*configs.numTasks);

for t = 1:configs.numTasks

    % The learning matrix
    learning_matrix(:,(t-1)*num_learning_subjs+1:(t-1)*num_learning_subjs+num_learning_subjs) = ...
        FCt_test_vec(:,(t-1)*configs.numSubjs+index_learning);
    labels_learning(1,(t-1)*num_learning_subjs+1:(t-1)*num_learning_subjs+num_learning_subjs) = t;

    % Within-Learning-Subject validation matrix and labels
    validation_matrix_within(:,(t-1)*num_learning_subjs+1:(t-1)*num_learning_subjs+num_learning_subjs) = ...
        FCt_retest_vec(:,(t-1)*configs.numSubjs+index_learning);
    labels_validation_within(1,(t-1)*num_learning_subjs+1:(t-1)*num_learning_subjs+num_learning_subjs) = t;
    
    % Across-Task validation matrix and labels
    validation_matrix_different(:,(t-1)*num_remaining_subjs+1:(t-1)*num_remaining_subjs+num_remaining_subjs) = ...
        FCt_retest_vec(:,(t-1)*configs.numSubjs+index_remaining_subjs);
    labels_validation_different(1,(t-1)*num_remaining_subjs+1:(t-1)*num_remaining_subjs+num_remaining_subjs) = t;

end

% Compute Task-Identification (TI) rates using Orig FCs and GEFF
% Within-Learning-Subject Task Identification
[rate_orig_within,CM_orig_within] = f_identify_task_orig_FCs(learning_matrix,validation_matrix_within,labels_learning,labels_validation_within,configs);
[rate_GEFF_within,CM_GEFF_within,rate_null_GEFF_within,CM_null_GEFF_within] = ...
    f_identify_task_GEFF(learning_matrix,validation_matrix_within,labels_learning,labels_validation_within,configs);

% Different-Subjects Task Identification
[rate_orig_different,CM_orig_different] = f_identify_task_orig_FCs(learning_matrix,validation_matrix_different,labels_learning,labels_validation_different,configs);
[rate_GEFF_different,CM_GEFF_different,rate_null_GEFF_different,CM_null_GEFF_different] = ...
    f_identify_task_GEFF(learning_matrix,validation_matrix_different,labels_learning,labels_validation_different,configs);

% Plot the figure
cmap = [0 0.4470 0.7410;
        0.8500 0.3250 0.0980;
        0.9290 0.6940 0.1250;
        0.4660 0.6740 0.1880];
    
h = figure('Name','Task Identification Curves','Units','normalized','Position',[0 0 1 1]);

% Within-Learning-Subjects TI rate curves
subplot(1,2,1);
x = 1:size(rate_GEFF_within,1); % number of eigenvectors/components
rate_avg = mean(rate_GEFF_within,2)'; % average across the three tasks
rate_sem = std(rate_GEFF_within,0,2)'/sqrt(size(rate_GEFF_within,2)); % standard error of mean across the tasks

f = fill([x,fliplr(x)],[rate_avg-rate_sem,fliplr(rate_avg+rate_sem)],cmap(1,:),'EdgeColor','none'); hold on;
alpha(f,0.25); % specifies the transparency of fill
hGEFF = plot(x,rate_avg,'Color',cmap(1,:));

% GEFF Null Model
clear rate_avg rate_sem
rate_avg = mean(rate_null_GEFF_within,2)'; % average across the three tasks
rate_sem = std(rate_null_GEFF_within,0,2)'/sqrt(size(rate_GEFF_within,2)); % standard error of mean across the tasks

f = fill([x,fliplr(x)],[rate_avg-rate_sem,fliplr(rate_avg+rate_sem)],cmap(4,:),'EdgeColor','none'); hold on;
alpha(f,0.25); % specifies the transparency of fill
hGEFF_null = plot(x,rate_avg,'Color',cmap(4,:));

% Orig FCs
clear rate_avg rate_sem
rate_avg = mean(rate_orig_within,2);
rate_sem = std(rate_orig_within,0,2)/sqrt(size(rate_orig_within,2));

f = fill([x,fliplr(x)],...
    [(rate_avg-rate_sem)*ones(1,length(x)),fliplr(rate_avg+rate_sem)*ones(1,length(x))],cmap(2,:),'EdgeColor','none');
alpha(f,0.25); % specifies the transparency of fill
hOrigFCs = plot(x,rate_avg*ones(1,length(x)),'Color',cmap(2,:));

axis([1 size(rate_GEFF_within,1) 0 1]);
xlabel('Eigenspace Dimensionality');
ylabel('Task Identification Rate');
title('Within-Learning-Subjects');
legend([hGEFF,hGEFF_null,hOrigFCs],'GEFF','Null Model (GEFF)','Orig FCs','Location','southeast');


% Different Subjects TI rate curves
subplot(1,2,2);
x = 1:size(rate_GEFF_different,1); % number of eigenvectors/components
rate_avg = mean(rate_GEFF_different,2)'; % average across the three tasks
rate_sem = std(rate_GEFF_different,0,2)'/sqrt(size(rate_GEFF_different,2)); % standard error of mean across the tasks

f = fill([x,fliplr(x)],[rate_avg-rate_sem,fliplr(rate_avg+rate_sem)],cmap(1,:),'EdgeColor','none'); hold on;
alpha(f,0.25); % specifies the transparency of fill
hGEFF = plot(x,rate_avg,'Color',cmap(1,:));

% GEFF Null Model
clear rate_avg rate_sem
rate_avg = mean(rate_null_GEFF_different,2)'; % average across the three tasks
rate_sem = std(rate_null_GEFF_different,0,2)'/sqrt(size(rate_GEFF_different,2)); % standard error of mean across the tasks

f = fill([x,fliplr(x)],[rate_avg-rate_sem,fliplr(rate_avg+rate_sem)],cmap(4,:),'EdgeColor','none'); hold on;
alpha(f,0.25); % specifies the transparency of fill
hGEFF_null = plot(x,rate_avg,'Color',cmap(4,:));

% Orig FCs
clear rate_avg rate_sem
rate_avg = mean(rate_orig_different,2);
rate_sem = std(rate_orig_different,0,2)/sqrt(size(rate_orig_different,2));

f = fill([x,fliplr(x)],...
    [(rate_avg-rate_sem)*ones(1,length(x)),fliplr(rate_avg+rate_sem)*ones(1,length(x))],cmap(2,:),'EdgeColor','none');
alpha(f,0.25); % specifies the transparency of fill
hOrigFCs = plot(x,rate_avg*ones(1,length(x)),'Color',cmap(2,:));

axis([1 size(rate_GEFF_different,1) 0 1]);
xlabel('Eigenspace Dimensionality');
ylabel('Task Identification Rate');
title('Different-Subjects');
legend([hGEFF,hGEFF_null,hOrigFCs],'GEFF','Null Model (GEFF)','Orig FCs','Location','southeast');

% Confusion Matrics
h = figure('Name','Task Identification Confusion Matrices','Units','normalized','Position',[0 0 0.5 1]);

% Orig FCs
subplot(2,2,1);
CM = CM_orig_within;
hh = heatmap(configs.Task_labels,configs.Task_labels,CM); colorbar;
title('Within-Learning-Subjects');
ylabel('Orig FCs');

subplot(2,2,2);
CM = CM_orig_different;
heatmap(configs.Task_labels,configs.Task_labels,CM); colorbar;
title('Different-Subjects');

% GEFF
subplot(2,2,3);
% Choose confusion matrix corresponding to the maximum number of
% eigenvectors/components
CM = squeeze(CM_GEFF_within(end,:,:));
heatmap(configs.Task_labels,configs.Task_labels,CM); colorbar;
ylabel('GEFF');

subplot(2,2,4);
% Choose confusion matrix corresponding to the maximum number of
% eigenvectors/components
CM = squeeze(CM_GEFF_different(end,:,:));
heatmap(configs.Task_labels,configs.Task_labels,CM); colorbar;

%% Characterization of Individual Eigenvectors
% Create a matrix with test and retest FCs for all available tasks
% First half of FCs would be test FCs, while the second half would be retest FCs
% Within each half, FCs would be organized by task

FC_all_vec = [FCt_test_vec FCt_retest_vec];

% Computation of Eigenspace representations
[COEFF,~,LATENT] = pca(FC_all_vec);
max_num_comps = size(COEFF,2);
var = LATENT./sum(LATENT);

% Create grouping variables for ANOVA
g_all = ones(1,max_num_comps);

Task_nums = 1:configs.numTasks;
tmp = repelem(Task_nums,configs.numSubjs);
g_task = [tmp tmp];

tmp = 1:configs.numSubjs;
g_subj = repmat(tmp,[1,2*configs.numTasks]);

for i = 1:max_num_comps
    y = COEFF(:,i);
    [p(:,i),tbl{i},stats{i},terms{i}] = anovan(y,{g_task g_subj},'random',2,'model','interaction','display','off');
    task_stat(i) = tbl{i}{2,6}; % F-stat for task-factor
    subj_stat(i) = tbl{i}{3,6}; % F-state for subject-factor
end

h = figure('Name','N-way ANOVA','Units','normalized','Position',[0 0 1 1]);
x = 1:max_num_comps;
threshold = 0.01/max_num_comps; % significance level
MarkerSize = 20;
dotColor = [0.3 0.3 0.3];

% Explained Variance of each component
subplot(3,1,1);
semilogx(x,var,'o','MarkerFaceColor','k','MarkerEdgeColor','k','MarkerSize',MarkerSize); grid on;
xlim([1 max_num_comps]);
ylabel('Explained Variance','FontSize',20,'FontWeight','bold');

subplot(3,1,2);
mask_sig = p(1,:) < threshold;
semilogx(x(mask_sig),task_stat(mask_sig),'o','MarkerFaceColor',dotColor,...
    'MarkerEdgeColor',[0.8500 0.3250 0.0980],'LineWidth',2,'MarkerSize',MarkerSize+5); grid on; hold on;
semilogx(x(~mask_sig),task_stat(~mask_sig),'o','MarkerFaceColor',dotColor,...
    'MarkerEdgeColor',dotColor,'MarkerSize',MarkerSize);
ylabel('Task-Specificity','FontSize',20,'FontWeight','bold');

subplot(3,1,3);
mask_sig = p(2,:) < threshold;
semilogx(x(mask_sig),subj_stat(mask_sig),'o','MarkerFaceColor',dotColor,...
    'MarkerEdgeColor',[0.8500 0.3250 0.0980],'LineWidth',2,'MarkerSize',MarkerSize+5); grid on; hold on;
semilogx(x(~mask_sig),subj_stat(~mask_sig),'o','MarkerFaceColor',dotColor,...
    'MarkerEdgeColor',dotColor,'MarkerSize',MarkerSize);
xlim([1 max_num_comps]); 
ylabel('Subject-Specificity','FontSize',20,'FontWeight','bold');
xlabel('Eigenspace Dimensionality','FontSize',20,'FontWeight','bold')



















































